package com.brainysoftware.batik;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Ellipse2D;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.batik.dom.svg.SVGDOMImplementation;
import org.apache.batik.dom.util.DOMUtilities;
import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.batik.swing.JSVGCanvas;
import org.apache.batik.transcoder.Transcoder;
import org.apache.batik.transcoder.TranscoderException;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.JPEGTranscoder;
import org.apache.batik.transcoder.image.PNGTranscoder;
import org.apache.batik.transcoder.image.TIFFTranscoder;
import org.apache.batik.util.SVGConstants;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Element;
import org.w3c.dom.svg.SVGDocument;

public class CreatorPlus2T extends JFrame 
		implements ActionListener, ChangeListener {
	static final long serialVersionUID = 1111;
	
	// Two Buttons to draw the shapes
	private JButton btnCircle, btnSquare, btnSaveSVG, btnSaveJPG,
		btnSavePNG, btnSaveTIFF; 
	
	// File Chooser to name and place the file being saved
	// in a file system
	JFileChooser fcSave = new JFileChooser();
	
	// Two Spinners to change the stroke width and the shapes' size
	private JSpinner spStroke, spSize;
	
	// The SVG generator
	private SVGGraphics2D generator;
	
	// The canvas to display results
	private JSVGCanvas canvas;
	
	// The SVGDocument is used twice: first to create an instance of
	// SVG Generator and then to pass the created SVG content to the
	// JSVGCanvas
	private SVGDocument doc;
	
	// We shall use Spinners to change these values
	private float strokeWidth = 20;
	private int figureSize = 90;
	
	// Constructor
	public CreatorPlus2T() {
		// Everything as usual here
		super("SVG Creator Plus 2T");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JPanel panel = new JPanel();
		
		// Auxilliary panels to group together Buttons, Labels and 
		// Spinners
		JPanel pTop = new JPanel();
		JPanel pBottom = new JPanel();
		
		// And here the canvas is instantiated
		canvas = new JSVGCanvas();
		
		// As before, the buttons are created and associated
		// with an Action Listener
		btnCircle = new JButton("Circle");
		btnSquare = new JButton("Square");
		btnSaveSVG = new JButton("SVG");
		btnSaveJPG = new JButton("JPG");
		btnSavePNG = new JButton("PNG");
		btnSaveTIFF = new JButton("TIFF");
		btnCircle.addActionListener(this);
		btnSquare.addActionListener(this);
		btnSaveSVG.addActionListener(this);
		btnSaveJPG.addActionListener(this);
		btnSavePNG.addActionListener(this);
		btnSaveTIFF.addActionListener(this);
		
		// Labels
		JLabel lblStrokeWidth = new JLabel("Stroke Width:");
		JLabel lblFigureSize = new JLabel("Figure Size:");
		JLabel lblSaveAs = new JLabel("Save As: ");
		
		// And then the Spinners themselves are created and associated
		// with a Change Listener
		SpinnerModel widthModel = new SpinnerNumberModel(20, 10, 50, 10);
		spStroke = new JSpinner(widthModel);
		spStroke.addChangeListener(this);		
		SpinnerModel sizeModel = new SpinnerNumberModel(90, 60, 110, 10);
		spSize = new JSpinner(sizeModel);
		spSize.addChangeListener(this);
		
		// Controls are added to the top auxiliary panel
		pTop.add(lblStrokeWidth);
		pTop.add(spStroke);
		pTop.add(lblFigureSize);
		pTop.add(spSize);
		pTop.add(btnCircle);
		pTop.add(btnSquare);
		
		// And here the bottom auxiliary panel is filled with components
		pBottom.add(lblSaveAs);
		pBottom.add(btnSaveSVG);
		pBottom.add(btnSaveJPG);
		pBottom.add(btnSavePNG);
		pBottom.add(btnSaveTIFF);
			
		// The main panel is filled with content and ready
		// to be displayed
		panel.setLayout(new BorderLayout());
		panel.add("North", pTop);
		panel.add("Center", canvas);
		panel.add("South", pBottom);
		this.setContentPane(panel);
		this.pack();
		this.setBounds(150, 150, this.getWidth(), this.getHeight());
		
		// SVGDocument and SVG Generator are created
		DOMImplementation dom = SVGDOMImplementation.getDOMImplementation();
		String svgNS = SVGDOMImplementation.SVG_NAMESPACE_URI;
		doc = (SVGDocument) dom.createDocument(svgNS, "svg", null);
		generator = new SVGGraphics2D(doc); 
		
		// There is no content at this stage, but placing this method call here
		// allows to avoid latency when drawing the first shape
		passContent();
	}
	
	// The entry point for running the program
	public static void main(String[] args) {
		CreatorPlus2T creator = new CreatorPlus2T();
		creator.setVisible(true);
	}

	public void actionPerformed(ActionEvent ae) {
		Object source = ae.getSource();
		
		// Randomly changing the location of shapes
		int x = (int)(300 * Math.random());
		int y = (int)(100 * Math.random());
		
		// Drawing a shape depending on which button was pressed
		if (source == btnCircle) {
			drawCircle(x,y);
			// Passing the created content to JSVGCanvas for displaying
			passContent();
		}
		else if (source == btnSquare) {
			drawSquare(x,y);	
			// Passing the created content to JSVGCanvas for displaying
			passContent();
		}
		else { // One of the saving buttons was pressed
			int returnVal = fcSave.showSaveDialog(this);
		    if (returnVal == JFileChooser.APPROVE_OPTION) {	
		    	File file = fcSave.getSelectedFile();
		    	
		    	// Depending on which button was pressed, method call
		    	// will be different
		    	if (source == btnSaveSVG) saveToSVGFile(file.getPath());
		    	if (source == btnSaveJPG) saveTranscode('j', file.getPath());
		    	if (source == btnSavePNG) saveTranscode('p', file.getPath());
		    	if (source == btnSaveTIFF) saveTranscode('t', file.getPath());
		    }

		}
	}
	
	private void passContent() {
		// And this is where the generated content is passed 
		// to the JSVGCanvas
		Element root = doc.getDocumentElement();
		generator.getRoot(root);
		canvas.setSVGDocument(doc);
	}
	
	// Changing the stroke and the size, basic Spinner usage 
	public void stateChanged(ChangeEvent e) {
		JSpinner source = (JSpinner)e.getSource();
		if (source == spStroke) {
			SpinnerNumberModel model = (SpinnerNumberModel) source.getModel();
			strokeWidth = ((Integer)model.getValue()).intValue();
		}
		if (source == spSize) {
			SpinnerNumberModel model = (SpinnerNumberModel) source.getModel();
			figureSize = ((Integer)model.getValue()).intValue();
		}
	}

	// Drawing itself, only slightly different from what was done in the previous 
	// application
	private void drawSquare(int x, int y) {
		Rectangle rect = new Rectangle(x, y, figureSize, figureSize);
		generator.setPaint(Color.red);
		generator.setStroke(new BasicStroke(strokeWidth));
		generator.draw(rect);
		generator.setPaint(Color.lightGray);
		generator.fill(rect);
	}
	
	private void drawCircle(int x, int y) {
		Ellipse2D circle = new Ellipse2D.Double(x, y, figureSize, figureSize);
		generator.setPaint(Color.green);
		generator.setStroke(new BasicStroke(strokeWidth));
		generator.draw(circle);
		generator.setPaint(Color.yellow);
		generator.fill(circle);		
	}
	
	// Saving as an SVG document
	private void saveToSVGFile(String file) {
		try {
			FileWriter f = new FileWriter(file);
			PrintWriter writer = new PrintWriter(f);
			writer.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
            writer.write ("<!DOCTYPE svg PUBLIC '");
            writer.write (SVGConstants.SVG_PUBLIC_ID);
            writer.write ("' '");
            writer.write (SVGConstants.SVG_SYSTEM_ID);
            writer.write ("'>\n\n");
			SVGDocument svgDoc = canvas.getSVGDocument();
			DOMUtilities.writeDocument(svgDoc, writer);
			writer.close();
		} catch (IOException ioe) {
			System.err.println("IO problem: " + ioe.toString());
		}
	}
	
	// Saving as a raster file
	private void saveTranscode(char format, String file) {
		try {
			OutputStream ostream = new FileOutputStream(file);
			Transcoder t = null;
			if (format == 'j') { // Save as a JPEG file
				t = new JPEGTranscoder();
				
				// Set the quality
				t.addTranscodingHint(JPEGTranscoder.KEY_QUALITY, new Float(.8));
			}
			if (format == 'p') {
				t = new PNGTranscoder();
				
				// Set the number of colors used in palette
				t.addTranscodingHint(PNGTranscoder.KEY_INDEXED, new Integer(5));
			}
			if (format == 't') {
				t = new TIFFTranscoder();
				
				// Set the width of the image
				t.addTranscodingHint(TIFFTranscoder.KEY_WIDTH, new Float(200));
			}
			if (t == null) return;
			
			// Creating input and output objects
			TranscoderInput input = new TranscoderInput(canvas.getSVGDocument());
			TranscoderOutput output = new TranscoderOutput(ostream);
			
			// Transcoding
			t.transcode(input, output);
			
			ostream.flush();
			ostream.close();
		} 
		catch (TranscoderException te) {
			System.err.println("Transcoder problem: " + te.toString());
		}
		catch (IOException ioe) {
			System.err.println("IO problem: " + ioe.toString());
		}
	}
}
